#include <exec/types.h>

BOOL use_RCS = FALSE;

#include "rcsio.h"
#include <exec/memory.h>
#include <exec/nodes.h>
#include <libraries/dos.h>
#include <libraries/dosextens.h>
#include <stdlib.h>
#include <string.h>
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/dos_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/dos.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/dos.h>
#endif

/*
// NAME:    rename_revision - rename a file to a revision level.
//
// SYNTAX:
//          rename_revision(  char* filename );
//
// SYNOPSIS:
//
//    Given filename 'filename' of an existing file, rename
//    it to 'filename,#', where '#' is a revision level.
//
//    '#' starts at 1.  If 'filename,#' already exists, then
//    '#' is incremented and retried.  r can range from 1 to 999.
//
// NOTES:   This is for systems w/o RCS.
//
// AUTHOR:  Lee Willis
*/

long
rename_revision( char *filename )
{
   char *revname;
   long result = -1;

   char* make_revname ( char *filename );

   if( revname = make_revname( filename ) )
   {
      result = Rename( filename, revname );
      free( revname );
   }

   return result;
}



/*
// NAME: make_revname - create a 'revision name' for 'filename'
//
// SYNOPSIS:
//
//    Given an existing file named 'filename', generate a string
//    of the form 'filename,#' where '#' is a revision number.
//
//    'filename,#' must not already exists as a file.  If it does,
//    '#' is incremented, and retried.
//
// CALLED BY:  rename_revision.
//
// AUTHOR: Lee Willis
*/

char*
make_revname( char *filename )
{
   int len;
   int rev;
   struct Lock *lock;
   char *revname;

   len = strlen( filename ) +1 + 5;

   revname = malloc( len );

   for( rev = 1; rev < 1000; rev++ )
   {
      revname[0] = '\0';
      sprintf( revname, "%s,%d", filename, rev );

      lock = (struct Lock *)Lock( revname, ACCESS_READ );

      if( lock == NULL ) /* This revision does not exist already. */
      {
         return revname;
      }

      UnLock( (BPTR)lock );
   }

   free( revname );
   return NULL;
}




typedef struct rcsfile
{
   FILE *file;
   char *name;
   ULONG flags;
} rcsfile;

#define PREEXISTS_AS_FILE 1
#define PREEXISTS_IN_RCS  2
#define PREEXISTS_AS_BOTH (PREEXISTS_AS_FILE | PREEXISTS_IN_RCS)
#define PREEXISTS_NEITHER 0
#define PREEXISTS_MASK    PREEXISTS_BOTH

/*
// PREEXISTS_AS_FILE is set if this file already existed in the directory.
//
// PREEXISTS_IN_RCS  is set if this file already existed in RCS.
//
// PREEXISTS_BOTH
*/


rcsfile *
rcs_new( char *name )
{
   rcsfile *f;

   if( f = malloc( sizeof( rcsfile ) ) )
   {
      f->file  = NULL;
      f->name  = strdup( name );
      f->flags = 0;
   }

   return f;
}


void
rcs_free( rcsfile *file )
{
   if( file->name ) free( file->name );
   free( file );
}




void
rcs_backup( rcsfile *rfile )
{
   struct Lock *lock;

   lock = (struct Lock *)Lock( rfile->name, ACCESS_READ );

   if( lock ) /* filename exists. */
   {
      rfile->flags |= PREEXISTS_AS_FILE;
      UnLock( (BPTR)lock );
   }

#ifdef NOT_FINISHED_YET
   if( use_RCS )
   {
      rfile->flags |= ( rcs_preexists( rfile->name ) ? PREEXISTS_IN_RCS : 0 );

      switch( rfile->flags & PREEXISTS_MASK )
      {
         case PREEXISTS_NEITHER:
            /* Do nothing. */
            break;

         case PREEXISTS_IN_RCS:
            rcs_checkout_lock( rfile->name );
            break;

         case PREEXISTS_AS_FILE:
            /* check into RCS initially. */
            break;

         case PREEXISTS_AS_BOTH:
            /* check new rev. into RCS. */
            rcs_checkin_checkout_lock_quiet( rfile->name );
            break;
      }
   }
   else /* RCS is not to be used. :-( */
#endif

   {
      if( rfile->flags & PREEXISTS_AS_FILE )
         rename_revision( rfile->name );
   }

}



/*
   Actions to perform before creating new file:

                       Assume
      File exists in | RCS file |
       Dir     RCS   | locked?  | Action
      ------+--------+----------+--------------------------------------
       No   |  No    |  N/A     | No action.
       No   |  Yes   |  No      | Check-out w/lock file.
       Yes  |  No    |  N/A     | Check file into RCS as initial entry.
       Yes  |  Yes   |  Yes(?)  | Check file into RCS as new revision.


   Actions after closing a file:

      File exists in |
       Dir     RCS   |  Action
      ------+--------+----------------------------------------
       No   |  No    |  Check file into RCS as initial entry.
       No   |  Yes   |  Check file into RCS as new revision.
       Yes  |  No    |  Check file into RCS as new revision.
       Yes  |  Yes   |  Check file into RCS as new revision.

*/



RCSFILE*
rcs_create( char *name )
{
   rcsfile *f;

   if( f = rcs_new( name ) )
   {
      rcs_backup( f );

      if( f->file = fopen( name, "w" ) )
      {
         ;
      }
      else
      {
         rcs_free( f );
         f = NULL;
      }
   }

   return (RCSFILE *)f;
}



void
rcs_close( RCSFILE *file )
{
   rcsfile *rf = (rcsfile *)file;

   if( rf )
   {
      fclose( rf->file );
      rcs_free( rf );
   }
}
