#ifndef RCSIO_H
#define RCSIO_H

/*
// MODULE:  RCSio - File writing functions which don't clobber
//                  previous versions of user's files.
//
// SYNOPSIS:
//
//    These functions provide an easy way of writing files
//    without deleting any pre-existing versions of those files.
//
//    Instead of opening/closing files with 'fopen()' and 'fclose()',
//    use 'rcs_open()' and 'rcs_close()'.
//
//    These functions work whether or not RCS is available on the system.
//
//    If RCS is available, existing files are checked into RCS before
//    overwriting.
//
//    If RCS is not available, existing files are renamed before
//    overwriting.
//
// AUTHOR:  Lee Willis
*/

#include <exec/types.h>
#include <stdio.h>

extern BOOL use_RCS;
   /*
   // If set to TRUE, RCS is available on the user's system.
   //
   // Defaults to FALSE.  If FALSE, file-renaming is used to
   // prevent destruction of files.
   */


typedef struct { FILE *file; } RCSFILE;
   /*
   // One obtains this structure from rcs_create.
   //
   // There is addition hidden data in here, so the _only_ legal
   // way of getting an RCSFILE is through 'rcs_create()'.
   //
   // Once you obtain the structure, its legal to use RCSFILE->file
   // in stdio functions.
   */



/*
// FUNCTION: rcs_create - create a new file, preserving any prior file
//                        of the same name.
//
// SYNOPSIS:
//
//    The behavior of rcs_create() varies depending if you've
//    set 'use_RCS' or not.:
//
//    If use_RCS == FALSE:
//
//       If file 'name' does not already exist, no special action
//       is taken.
//
//       If 'name' does exist, then the existing version of 'name'
//       is renamed to 'name,1'.  (if 'name,1' also exists, then
//       'name' becomes 'name,2' or 'name,3', or ...)
//
//    If use_RCS == TRUE: Its a little more complex:
//
//       'name' exists | 'name' exists |
//       as a file     | within RCS    |  Action
//       ==============+===============+==========================
//          NO         |      NO       |  None.
//       --------------+---------------+--------------------------
//          NO         |      YES      |  Check-out/lock 'name' from RCS.
//       --------------+---------------+--------------------------
//          YES        |      NO       |  Check-in 'name' into RCS
//                     |               |  as a new file.
//       --------------+---------------+--------------------------
//          YES        |      YES      |  Checkin 'name' into RCS
//                     |               |  as a new revision, and check
//                     |               |  'name' out again. ("ci -l")
//
// AUTHOR:  Lee Willis
*/

RCSFILE* rcs_create ( char *name );

void     rcs_close ( RCSFILE *file );

#endif

